// ignore_for_file: close_sinks

import 'dart:async';
import 'package:pam_flutter/repositories/types.dart';
import 'package:rxdart/rxdart.dart';

class BaseDataRepository<T> implements IBaseDataRepository {
  final IRepositoryOptions options;

  bool _isLoading = false;
  bool _isLoaded = false;
  bool _isError = false;
  String _errorMessage = "";
  List<T>? items;
  T? data;

  late StreamController<bool> _isLoadingSC;
  late StreamController<bool> _isLoadedSC;
  late StreamController<bool> _isErrorSC;
  late StreamController<String> _errorMessageSC;
  late StreamController<List<T>?> _itemsSC;
  late StreamController<T?> _dataSC;

  BaseDataRepository(
    this.options,
  ) {
    this.initial();
  }

  @override
  bool get isLoading => this._isLoading;

  @override
  bool get isLoaded => this._isLoaded;

  @override
  bool get isError => this._isError;

  @override
  String get errorMessage => this._errorMessage;

  @override
  StreamController<bool> get isLoadingSC => this._isLoadingSC;

  @override
  StreamController<bool> get isLoadedSC => this._isLoadedSC;

  @override
  StreamController<bool> get isErrorSC => this._isErrorSC;

  @override
  StreamController<String> get errorMessageSC => this._errorMessageSC;

  @override
  StreamController<List<T>?> get itemsSC => this._itemsSC;

  @override
  StreamController<T?> get dataSC => this._dataSC;

  @override
  void toLoadingStatus() {
    this._isLoading = true;
    this._isLoadingSC.add(true);

    this._isLoaded = false;
    this._isLoadedSC.add(false);

    this._isError = false;
    this._isErrorSC.add(false);
  }

  @override
  void toLoadedStatus() {
    this._isLoading = false;
    this._isLoadingSC.add(false);

    this._isLoaded = true;
    this._isLoadedSC.add(true);
  }

  @override
  void toErrorStatus(e) {
    this._isLoading = false;
    this._isLoadingSC.add(false);

    this._isError = true;
    this._isErrorSC.add(true);

    this._errorMessage = e.toString();
    this._errorMessageSC.add(e.toString());

    print("error: ${e.toString()}");
  }

  @override
  Future<void> fetch({Map<String, dynamic>? params, bool isMock: false}) async {
  }

  @override
  Future<void> fetchAfterId(String afterId,
      {Map<String, dynamic>? params}) async {
  }

  @override
  Future<void> get(String id,
      {Map<String, dynamic>? params}) async {
  }

  @override
  void forceValueNotify() {
    this._isLoadingSC.add(true);
    this._isLoadedSC.add(false);
    this._isErrorSC.add(false);
    this._errorMessageSC.add("");

    this._itemsSC.add(null);
    this._dataSC.add(null);

    this._isLoadingSC.add(this.isLoading);
    this._isLoadedSC.add(this.isLoaded);
    this._isErrorSC.add(this.isError);
    this._errorMessageSC.add(this.errorMessage);

    this._itemsSC.add(this.items);
    this._dataSC.add(this.data);
  }

  @override
  void initial() {
    this._isLoadingSC = BehaviorSubject<bool>();
    this._isLoadedSC = BehaviorSubject<bool>();
    this._isErrorSC = BehaviorSubject<bool>();
    this._errorMessageSC = BehaviorSubject<String>();

    this._itemsSC = BehaviorSubject<List<T>?>();
    this._dataSC = BehaviorSubject<T?>();
  }

  @override
  void dispose() {
    this._isLoadingSC.close();
    this._isLoadedSC.close();
    this._isErrorSC.close();
    this._errorMessageSC.close();
  }
}

class NewRepositoryOptions implements IRepositoryOptions {
  final String baseUrl;
  final String? addUrl;
  final String? updateUrl;
  final String? deleteUrl;
  final List<Map<String, dynamic>>? mockItems;
  final Map<String, dynamic>? mockItem;

  NewRepositoryOptions({
    required this.baseUrl,
    this.addUrl,
    this.updateUrl,
    this.deleteUrl,
    this.mockItems,
    this.mockItem,
  });

  @override
  String getBaseUrl() {
    return this.baseUrl;
  }

  @override
  String? getAddUrl() {
    return this.addUrl;
  }

  @override
  String? getUpdateUrl() {
    return this.updateUrl;
  }

  @override
  String? getDeleteUrl() {
    return this.deleteUrl;
  }

  @override
  List<Map<String, dynamic>>? getMockItems() {
    return this.mockItems;
  }

  @override
  Map<String, dynamic>? getMockItem() {
    return this.mockItem;
  }
}
