import 'dart:async';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:pam_flutter/models/consent_message_model.dart';
import 'package:pam_flutter/utils/time_helper.dart';
import 'package:pam_flutter/widgets/commons/switch_button.dart';

class ConsentItem extends StatefulWidget {
  final String title;
  final ConsentMessageSettingConsentDetailModel? setting;
  final void Function(bool isAllow) onAllowed;

  ConsentItem({
    required this.title,
    required this.setting,
    required this.onAllowed,
  });

  @override
  State<StatefulWidget> createState() {
    return ConsentItemState();
  }
}

class ConsentItemState extends State<ConsentItem> {
  late bool _isConsentExpand;
  late StreamController<bool> _isConsentExpandSC;

  late bool _isInnerConsentExpand;
  late StreamController<bool> _isInnerConsentExpandSC;

  @override
  void initState() {
    super.initState();

    _isConsentExpand = false;
    _isInnerConsentExpand = false;
    this._isConsentExpandSC = StreamController<bool>();
    this._isInnerConsentExpandSC = StreamController<bool>();

    this._isConsentExpandSC.add(this._isConsentExpand);
    this._isInnerConsentExpandSC.add(this._isInnerConsentExpand);
  }

  @override
  void dispose() {
    super.dispose();

    this._isConsentExpandSC.close();
    this._isInnerConsentExpandSC.close();
  }

  onConsentExpandClicked() {
    this._isConsentExpand = !this._isConsentExpand;
    this._isConsentExpandSC.add(this._isConsentExpand);

    if (this._isConsentExpand) {
      this.innerConsentExpandOpened();
    } else {
      this.innerConsentExpandClosed();
    }
  }

  Future<void> innerConsentExpandOpened() async {
    await TimeHelper.sleep(milliseconds: 50);
    this._isInnerConsentExpand = !this._isInnerConsentExpand;
    this._isInnerConsentExpandSC.add(this._isInnerConsentExpand);
  }

  Future<void> innerConsentExpandClosed() async {
    await TimeHelper.sleep(milliseconds: 50);
    this._isInnerConsentExpand = !this._isInnerConsentExpand;
    this._isInnerConsentExpandSC.add(this._isInnerConsentExpand);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Column(
        children: [
          GestureDetector(
            onTap: () {
              this.onConsentExpandClicked();
            },
            child: Container(
              color: Colors.grey.shade300,
              padding: EdgeInsets.only(
                top: 20,
                bottom: 20,
                left: 16,
                right: 16,
              ),
              child: Row(
                children: [
                  Container(
                    margin: EdgeInsets.only(right: 12),
                    child: Icon(
                      Icons.check_rounded,
                      color: Colors.green,
                      size: 18,
                    ),
                  ),
                  Expanded(
                    child: Container(
                      child: Text(
                        widget.title,
                        style: TextStyle(
                          fontSize: 18,
                          fontWeight: FontWeight.bold,
                        ),
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
          StreamBuilder<bool>(
            stream: this._isConsentExpandSC.stream,
            builder: (context, snapshot) {
              return AnimatedOpacity(
                duration: Duration(milliseconds: 100),
                opacity: snapshot.data == true ? 1 : 0,
                child: StreamBuilder<bool>(
                  stream: this._isInnerConsentExpandSC.stream,
                  builder: (context, innerSnapshot) {
                    return AnimatedContainer(
                      duration: Duration(milliseconds: 100),
                      padding: EdgeInsets.all(16),
                      height: snapshot.data == true ? 340 : 0,
                      child: Column(
                        children: [
                          Expanded(
                            child: Container(
                              child: Text(
                                innerSnapshot.data == true
                                    ? widget.setting?.BriefDescription.TH ?? ""
                                    : "",
                                maxLines: 14,
                                overflow: TextOverflow.ellipsis,
                              ),
                            ),
                          ),
                          innerSnapshot.data == true
                              ? Container(
                                  margin: EdgeInsets.only(top: 16),
                                  child: Row(
                                    mainAxisAlignment:
                                        MainAxisAlignment.spaceBetween,
                                    children: [
                                      Container(
                                        child: Text(
                                          "Full Version",
                                          style: TextStyle(
                                            color: Colors.grey.shade600,
                                            decoration:
                                                TextDecoration.underline,
                                          ),
                                        ),
                                      ),
                                      Container(
                                        child: SwitchButton(
                                          title: "Accept",
                                          defaultValue: widget.setting?.IsAllowed ?? false,
                                          onChanged: (v) {
                                            widget.onAllowed(v);
                                          },
                                          buttonColor: Colors.white,
                                          backgroundColor: Colors.green,
                                        ),
                                      ),
                                    ],
                                  ),
                                )
                              : Container(),
                        ],
                      ),
                    );
                  },
                ),
              );
            },
          ),
        ],
      ),
    );
  }
}
